@@ -19,11 +19,10 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-const { Meta, Shell, St } = imports.gi;
+const { Clutter, Meta, Shell, St } = imports.gi;
 
 const ExtensionUtils = imports.misc.extensionUtils;
 const Main = imports.ui.main;
-const Tweener = imports.ui.tweener;
 
 const MESSAGE_FADE_TIME = 2000;
 
@@ -65,17 +64,17 @@ let SIZES = [
     [1200, 675],
     [1600, 900],
     [360, 654], // Phone portrait maximized
-    [720, 360] // Phone landscape fullscreen
+    [720, 360], // Phone landscape fullscreen
 ];
 
 function cycleScreenshotSizes(display, window, binding) {
     // Probably this isn't useful with 5 sizes, but you can decrease instead
     // of increase by holding down shift.
     let modifiers = binding.get_modifiers();
-    let backwards = (modifiers & Meta.VirtualModifier.SHIFT_MASK) != 0;
+    let backwards = (modifiers & Meta.VirtualModifier.SHIFT_MASK) !== 0;
 
     // Unmaximize first
-    if (window.get_maximized() != 0)
+    if (window.get_maximized() !== 0)
         window.unmaximize(Meta.MaximizeFlags.BOTH);
 
     let workArea = window.get_work_area_current_monitor();
@@ -83,7 +82,8 @@ function cycleScreenshotSizes(display, window, binding) {
 
     // Double both axes if on a hidpi display
     let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
-    let scaledSizes = SIZES.map(size => size.map(wh => wh * scaleFactor));
+    let scaledSizes = SIZES.map(size => size.map(wh => wh * scaleFactor))
+        .filter(([w, h]) => w <= workArea.width && h <= workArea.height);
 
     // Find the nearest 16:9 size for the current window size
     let nearestIndex;
@@ -98,7 +98,7 @@ function cycleScreenshotSizes(display, window, binding) {
 
         // get the best initial window size
         let error = Math.abs(width - outerRect.width) + Math.abs(height - outerRect.height);
-        if (nearestIndex == null || error < nearestError) {
+        if (nearestIndex === undefined || error < nearestError) {
             nearestIndex = i;
             nearestError = error;
         }
@@ -106,10 +106,7 @@ function cycleScreenshotSizes(display, window, binding) {
 
     // get the next size up or down from ideal
     let newIndex = (nearestIndex + (backwards ? -1 : 1)) % scaledSizes.length;
-    let newWidth, newHeight;
-    [newWidth, newHeight] = scaledSizes[newIndex];
-    if (newWidth > workArea.width || newHeight > workArea.height)
-        [newWidth, newHeight] = scaledSizes[0];
+    let [newWidth, newHeight] = scaledSizes[newIndex];
 
     // Push the window onscreen if it would be resized offscreen
     let newX = outerRect.x;
@@ -123,13 +120,13 @@ function cycleScreenshotSizes(display, window, binding) {
 
     let newOuterRect = window.get_frame_rect();
     let message = '%d×%d'.format(
-        (newOuterRect.width / scaleFactor),
-        (newOuterRect.height / scaleFactor));
+        newOuterRect.width / scaleFactor,
+        newOuterRect.height / scaleFactor);
 
     // The new size might have been constrained by geometry hints (e.g. for
     // a terminal) - in that case, include the actual ratio to the message
     // we flash
-    let actualNumerator = (newOuterRect.width / newOuterRect.height) * 9;
+    let actualNumerator = 9 * newOuterRect.width / newOuterRect.height;
     if (Math.abs(actualNumerator - 16) > 0.01)
         message += ' (%.2f:9)'.format(actualNumerator);
 
