@@ -352,6 +352,9 @@ class WindowButton extends BaseButton {
         super._init(perMonitor, monitorIndex);
 
         this.metaWindow = metaWindow;
+        this._skipTaskbarId = metaWindow.connect('notify::skip-taskbar', () => {
+            this._updateVisibility();
+        });
         this._updateVisibility();
 
         this._windowTitle = new WindowTitle(this.metaWindow);
@@ -412,6 +415,7 @@ class WindowButton extends BaseButton {
 
     _onDestroy() {
         super._onDestroy();
+        this.metaWindow.disconnect(this._skipTaskbarId);
         this.metaWindow.disconnect(this._workspaceChangedId);
         global.display.disconnect(this._notifyFocusId);
         this._contextMenu.destroy();
@@ -783,9 +787,9 @@ class WindowList extends St.Widget {
             });
 
         this._dragBeginId = Main.xdndHandler.connect('drag-begin',
-            this._onDragBegin.bind(this));
+            this._monitorDrag.bind(this));
         this._dragEndId = Main.xdndHandler.connect('drag-end',
-            this._onDragEnd.bind(this));
+            this._stopMonitoringDrag.bind(this));
         this._dragMonitor = {
             dragMotion: this._onDragMotion.bind(this),
         };
@@ -948,9 +952,6 @@ class WindowList extends St.Widget {
     }
 
     _onWindowAdded(ws, win) {
-        if (win.skip_taskbar)
-            return;
-
         if (!this._grouped)
             this._checkGrouping();
 
@@ -1016,11 +1017,11 @@ class WindowList extends St.Widget {
         }
     }
 
-    _onDragBegin() {
+    _monitorDrag() {
         DND.addDragMonitor(this._dragMonitor);
     }
 
-    _onDragEnd() {
+    _stopMonitoringDrag() {
         DND.removeDragMonitor(this._dragMonitor);
         this._removeActivateTimeout();
     }
@@ -1094,6 +1095,7 @@ class WindowList extends St.Widget {
 
         global.display.disconnect(this._fullscreenChangedId);
 
+        this._stopMonitoringDrag();
         Main.xdndHandler.disconnect(this._dragBeginId);
         Main.xdndHandler.disconnect(this._dragEndId);
 
